@@ -1,15 +1,13 @@
 /**
  * @file   visual.c
- * @author Akinobu Lee
- * @date   Mon Sep 12 01:49:44 2005
  * 
  * <JA>
  * @brief  õ֤βĻ벽 (GTK/X11 )
  *
- * õֲĻ벽ǽ Linux ǥݡȤƤޤ
- * "--enable-visualize"  ON ˤƥѥǤޤ
- * ʤѤˤ gtk ΥС 1.x ɬפǤ
- * ưǧ gtk-1.2 ǹԤޤ
+ * õֲĻ벽ǽ Linux ǥݡȤƤޤ. 
+ * "--enable-visualize"  ON ˤƥѥǤޤ. 
+ * ʤѤˤ gtk ΥС 1.x ɬפǤ. 
+ * ưǧ gtk-1.2 ǹԤޤ. 
  * </JA>
  * 
  * <EN>
@@ -20,16 +18,20 @@
  * It needs gtk version = 1.x, (tested on gtk-1.2).
  * </EN>
  * 
- * $Revision: 1.2 $
+ * @author Akinobu Lee
+ * @date   Mon Sep 12 01:49:44 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
  * Copyright (c) 2003-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-#include <julius.h>
+#include <libmain.h>
+#include "app.h"
 
 #ifdef VISUALIZE
 
@@ -65,6 +67,7 @@ static boolean sw_hypo = FALSE;		///< Y axis is hypothesis score
 /**********************************************************************/
 /* data to plot (1st pass / 2nd pass) */
 /**********************************************************************/
+static Recog *re;		///< Local pointer to the whole instance
 /* data to plot on 1st pass */
 static BACKTRELLIS *btlocal = NULL; ///< Local pointer to the word trellis
 /* data to plot on 2nd pass */
@@ -115,7 +118,7 @@ color_init()
   
   defcolmap = gdk_colormap_get_system();
   if (gdk_colormap_alloc_colors(defcolmap, cols, NCOLS, FALSE, TRUE, success) > 0) {
-    j_printerr("Warning: some colors are not allocated\n");
+    fprintf(stderr, "Warning: some colors are not allocated\n");
   }
 
 }
@@ -133,7 +136,7 @@ static LOGPROB maxrange2;	///< Maximum distance from normalization line
 
 /** 
  * <JA>
- * Ѥ˺祹ȺǾ롥
+ * Ѥ˺祹ȺǾ. 
  * 
  * @param bt [in] ñȥꥹ
  * </JA>
@@ -187,11 +190,11 @@ get_max_frame_score(BACKTRELLIS *bt)
 
 /** 
  * <JA>
- * ֥ե졼 X ɸͤѴ롥
+ * ֥ե졼 X ɸͤѴ. 
  * 
  * @param t [in] ֥ե졼
  * 
- * @return б X ɸ֤ͤ
+ * @return б X ɸ֤ͤ. 
  * </JA>
  * <EN>
  * Scale X axis by time to fullfill in the canvas width.
@@ -209,12 +212,12 @@ scale_x(int t)
 
 /** 
  * <JA>
- *  Y ɸͤѴ롥
+ *  Y ɸͤѴ. 
  * 
  * @param s [in] 
  * @param t [in] б֥ե졼
  * 
- * @return Y ɸ֤ͤ
+ * @return Y ɸ֤ͤ. 
  * </JA>
  * <EN>
  * Scale Y axis from score to fulfill in the canvas height.
@@ -246,7 +249,7 @@ scale_y(LOGPROB s, int t)
     bottom = lowest * (float)t / (float)btlocal->framelen - maxrange2;
   }
 
-  yoffset = CANVAS_MARGIN + RESULT_HEIGHT + (speechlen != 0 ? (WAVE_MARGIN + WAVE_HEIGHT): 0);
+  yoffset = CANVAS_MARGIN + RESULT_HEIGHT + (re->speechlen != 0 ? (WAVE_MARGIN + WAVE_HEIGHT): 0);
   height = canvas_height - yoffset - CANVAS_MARGIN;
   if (top <= bottom) {	/* single or no token on the time */
     y = yoffset;
@@ -258,11 +261,11 @@ scale_y(LOGPROB s, int t)
 
 /** 
  * <JA>
- * ñID Y ɸͤѴ롥
+ * ñID Y ɸͤѴ. 
  * 
  * @param wid [in] ñID
  * 
- * @return Y ɸ֤ͤ
+ * @return Y ɸ֤ͤ. 
  * </JA>
  * <EN>
  * Scale Y axis from word id.
@@ -278,12 +281,12 @@ scale_y_wid(WORD_ID wid)
   gint y;
   gint yoffset, height;
   
-  yoffset = CANVAS_MARGIN + RESULT_HEIGHT + (speechlen != 0 ? (WAVE_MARGIN + WAVE_HEIGHT) : 0);
+  yoffset = CANVAS_MARGIN + RESULT_HEIGHT + (re->speechlen != 0 ? (WAVE_MARGIN + WAVE_HEIGHT) : 0);
   height = canvas_height - yoffset - CANVAS_MARGIN;
   if (wid == WORD_INVALID) {
     y = yoffset;
   } else {
-    y = wid * height / winfo->num + yoffset;
+    y = wid * height / re->model->winfo->num + yoffset;
   }
   return(y);
 }
@@ -296,11 +299,11 @@ static SP16 max_level;		///< Maximum level of input waveform
 
 /** 
  * <JA>
- * ȷɽѤ˻֤XɸѴ롥
+ * ȷɽѤ˻֤XɸѴ. 
  * 
  * @param t [in] ֥ե졼
  * 
- * @return Ѵ X ɸ֤
+ * @return Ѵ X ɸ֤. 
  * </JA>
  * <EN>
  * Scale time to X position.
@@ -313,16 +316,16 @@ static SP16 max_level;		///< Maximum level of input waveform
 static gint
 scale_x_wave(int t)
 {
-  return(t * (canvas_width - CANVAS_MARGIN * 2) / speechlen + CANVAS_MARGIN);
+  return(t * (canvas_width - CANVAS_MARGIN * 2) / re->speechlen + CANVAS_MARGIN);
 }
 
 /** 
  * <JA>
- * ȷɽѤ˿YɸѴ롥
+ * ȷɽѤ˿YɸѴ. 
  * 
  * @param x [in] ȷο
  * 
- * @return Ѵ X ɸ֤
+ * @return Ѵ X ɸ֤. 
  * </JA>
  * <EN>
  * Scale wave level to Y position
@@ -340,7 +343,7 @@ scale_y_wave(SP16 x)
 
 /** 
  * <JA>
- * κͤ speech[] 롥
+ * κͤ speech[] . 
  * 
  * </JA>
  * <EN>
@@ -354,12 +357,12 @@ get_max_waveform_level()
   int t;
   SP16 maxl;
   
-  if (speechlen == 0) return;	/* no waveform data (MFCC) */
+  if (re->speechlen == 0) return;	/* no waveform data (MFCC) */
   
   maxl = 0;
-  for(t=0;t<speechlen;t++) {
-    if (maxl < abs(speech[t])) {
-      maxl = abs(speech[t]);
+  for(t=0;t<re->speechlen;t++) {
+    if (maxl < abs(re->speech[t])) {
+      maxl = abs(re->speech[t]);
     }
   }
 
@@ -369,7 +372,7 @@ get_max_waveform_level()
 
 /** 
  * <JA>
- * ȷ speech[] 褹롥
+ * ȷ speech[] 褹. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -386,7 +389,7 @@ draw_waveform(GtkWidget *widget)
   gint text_width;
   static char buf[20];
 
-  if (speechlen == 0) return;	/* no waveform data (MFCC) */
+  if (re->speechlen == 0) return;	/* no waveform data (MFCC) */
 
   /* first time, make gc for drawing */
   if (dgc == NULL) {
@@ -399,19 +402,19 @@ draw_waveform(GtkWidget *widget)
   gdk_gc_set_foreground(dgc, &(cols[C_WAVEFORM]));
   gdk_draw_rectangle(pixmap, dgc, FALSE,
 		     scale_x_wave(0), scale_y_wave(max_level),
-		     scale_x_wave(speechlen-1) - scale_x_wave(0),
+		     scale_x_wave(re->speechlen-1) - scale_x_wave(0),
 		     scale_y_wave(-max_level) - scale_y_wave(max_level));
 
   if (sw_level_thres) {
     /* draw level threshold line */
     gdk_gc_set_foreground(dgc, &(cols[C_LEVELTHRES]));
     gdk_draw_line(pixmap, dgc,
-		  scale_x_wave(0), scale_y_wave(level_thres),
-		  scale_x_wave(speechlen-1), scale_y_wave(level_thres));
+		  scale_x_wave(0), scale_y_wave(re->jconf->detect.level_thres),
+		  scale_x_wave(re->speechlen-1), scale_y_wave(re->jconf->detect.level_thres));
     gdk_draw_line(pixmap, dgc,
-		  scale_x_wave(0), scale_y_wave(-level_thres),
-		  scale_x_wave(speechlen-1), scale_y_wave(-level_thres));
-    snprintf(buf, 20, "-lv %d", level_thres);
+		  scale_x_wave(0), scale_y_wave(- re->jconf->detect.level_thres),
+		  scale_x_wave(re->speechlen-1), scale_y_wave(- re->jconf->detect.level_thres));
+    snprintf(buf, 20, "-lv %d", re->jconf->detect.level_thres);
     text_width = gdk_string_width(fontset, buf) + 1;
     gdk_draw_string(pixmap, fontset, dgc,
 		    canvas_width - CANVAS_MARGIN - text_width - 2,
@@ -429,11 +432,11 @@ draw_waveform(GtkWidget *widget)
 		  buf);
 
   /* draw waveform */
-  for(t=1;t<speechlen;t++) {
+  for(t=1;t<re->speechlen;t++) {
     gdk_gc_set_line_attributes(dgc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
     gdk_draw_line(pixmap, dgc,
-		  scale_x_wave(t-1), scale_y_wave(speech[t-1]),
-		  scale_x_wave(t), scale_y_wave(speech[t]));
+		  scale_x_wave(t-1), scale_y_wave(re->speech[t-1]),
+		  scale_x_wave(t), scale_y_wave(re->speech[t]));
   }
 }
 
@@ -444,7 +447,7 @@ draw_waveform(GtkWidget *widget)
 
 /** 
  * <JA>
- * 褹롥
+ * 褹. 
  * 
  * @param widget [in] 襦å
  * @param x1 [in] 1Xɸ
@@ -536,7 +539,7 @@ mygdk_draw_arc(GtkWidget *widget, int x1, int y1, int x2, int y2, int sw)
 
 /** 
  * <JA>
- * ȥꥹñ褹륵ִؿ
+ * ȥꥹñ褹륵ִؿ. 
  * 
  * @param widget [in] 襦å
  * @param tre [in] 褹ȥꥹñ
@@ -616,7 +619,7 @@ draw_atom(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 /* draw word output text */
 /** 
  * <JA>
- * ȥꥹññɤߤ褹롥
+ * ȥꥹññɤߤ褹. 
  * 
  * @param widget [in] 襦å
  * @param tre [in] ȥꥹñ
@@ -637,6 +640,9 @@ draw_atom_text(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
   UserColors c;
   int style;
   int dx, dy, x, y;
+  WORD_INFO *winfo;
+
+  winfo = re->model->winfo;
   
   if (winfo->woutput[tre->wid] != NULL && strlen(winfo->woutput[tre->wid]) > 0) {
     switch(sw) {
@@ -684,11 +690,11 @@ static WORD_ID wordlistnum = 0;	///< Length of @a wordlist
 
 /** 
  * <JA>
- * ñꥹȤñ줬뤫ɤ롥
+ * ñꥹȤñ줬뤫ɤ. 
  * 
  * @param wid [in] ñID
  * 
- * @return ꥹȤ˸Ĥ TRUEĤʤ FALSE ֤
+ * @return ꥹȤ˸Ĥ TRUEĤʤ FALSE ֤. 
  * </JA>
  * <EN>
  * Check if the given word exists in the drawn word list.
@@ -727,7 +733,7 @@ wordlist_find(WORD_ID wid)
 
 /** 
  * <JA>
- * ñ褹ñꥹȤˤΤΤߡˡ
+ * ñ褹ñꥹȤˤΤΤߡ. 
  * 
  * @param widget [in] 襦å
  * @param tre [in] 褹ȥꥹñ
@@ -751,15 +757,15 @@ draw_atom_top(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 
 /** 
  * <JA>
- * ñ褹ñꥹȤˤΤΤߡˡ
- * ñΥƥȤ褹ʥꥹȤˤʤ褷ʤˡ
+ * ñ褹ñꥹȤˤΤΤߡ. 
+ * ñΥƥȤ褹ʥꥹȤˤʤ褷ʤ. 
  * 
  * @param widget [in] 襦å
  * @param tre [in] 褹ȥꥹñ
  * @param sw [in] ζ
  * </JA>
  * <EN>
- * ñΥƥȤ褹ñꥹȤˤΤΤߡˡ
+ * ñΥƥȤ褹ñꥹȤˤΤΤߡ. 
  * Draw text of a word on canvas (only words in wordlist).
  * 
  * @param widget [in] drawing widget
@@ -782,7 +788,7 @@ draw_atom_text_top(GtkWidget *widget, TRELLIS_ATOM *tre, int sw)
 
 /** 
  * <JA>
- * ƤΥȥꥹñ褹롥
+ * ƤΥȥꥹñ褹. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -816,7 +822,7 @@ draw_all_atom(GtkWidget *widget)
 
 /** 
  * <JA>
- * 1ѥˤƤμñ줬ĤäȥꥹñΤ褹롥
+ * 1ѥˤƤμñ줬ĤäȥꥹñΤ褹. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -897,7 +903,7 @@ draw_word_graph(GtkWidget *widget)
 
 /** 
  * <JA>
- * 1ѥΰʸ褹롥
+ * 1ѥΰʸ褹. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -919,7 +925,7 @@ draw_best_path(GtkWidget *widget)
   for (last_time = btlocal->framelen - 1; last_time >= 0; last_time--) {
 #ifdef USE_NGRAM
     /* it is fixed to the tail silence model (winfo->tail_silwid) */
-    last_tre = bt_binsearch_atom(btlocal, last_time, winfo->tail_silwid);
+    last_tre = bt_binsearch_atom(btlocal, last_time, re->model->winfo->tail_silwid);
     if (last_tre != NULL) break;
 #else /* USE_DFA */
     /* the best trellis word on the last frame (not use cp_end[]) */
@@ -964,7 +970,7 @@ draw_best_path(GtkWidget *widget)
 
 /** 
  * <JA>
- * 2ѥĻ벽ΤνԤ
+ * 2ѥĻ벽ΤνԤ. 
  * 
  * @param maxhypo [in] 2ѥˤƥݥåפ줿κ
  * </JA>
@@ -1011,7 +1017,7 @@ visual2_init(int maxhypo)
 
 /** 
  * <JA>
- * åФ¸롥
+ * åФ¸. 
  * 
  * @param n [in] 
  * @param popctr [in] ߤΥݥå׿
@@ -1038,7 +1044,7 @@ visual2_popped(NODE *n, int popctr)
   
 /** 
  * <JA>
- * 줿ΥΡɤ¸롥
+ * 줿ΥΡɤ¸. 
  * 
  * @param next [in] 줿ñ첾
  * @param prev [in] Ÿβ
@@ -1071,7 +1077,7 @@ visual2_next_word(NODE *next, NODE *prev, int popctr)
 
 /** 
  * <JA>
- * ݥåפ줿¸롥
+ * ݥåפ줿¸. 
  * 
  * @param now [in] ʸ
  * @param winfo [in] ñ켭
@@ -1103,7 +1109,7 @@ visual2_best(NODE *now, WORD_INFO *winfo)
 /* draw 2nd pass results */
 /** 
  * <JA>
- * 2ѥõˡåФ줿Ȥμñ콸褹롥
+ * 2ѥõˡåФ줿Ȥμñ콸褹. 
  * 
  * @param widget 襦å
  * </JA>
@@ -1137,7 +1143,7 @@ static LOGPROB maxscore;	///< Maximum score of popped hypotheses while search
 static LOGPROB minscore;	///< Minimum score of popped hypotheses while search
 /** 
  * <JA>
- * 2ѥǽиŸΥκͤȺǾͤ롥
+ * 2ѥǽиŸΥκͤȺǾͤ. 
  * 
  * </JA>
  * <EN>
@@ -1161,7 +1167,7 @@ get_max_hypo_score()
 
 /** 
  * <JA>
- * ɽ⥹ Y ɸͤѴ롥
+ * ɽ⥹ Y ɸͤѴ. 
  * 
  * @param s [in] ⥹
  * 
@@ -1181,7 +1187,7 @@ scale_hypo_y(LOGPROB s)
   gint y;
   gint yoffset, height;
 
-  yoffset = CANVAS_MARGIN + RESULT_HEIGHT + (speechlen != 0 ? (WAVE_MARGIN + WAVE_HEIGHT) : 0);
+  yoffset = CANVAS_MARGIN + RESULT_HEIGHT + (re->speechlen != 0 ? (WAVE_MARGIN + WAVE_HEIGHT) : 0);
   height = canvas_height - yoffset - CANVAS_MARGIN;
   y = (maxscore - s) * height / (maxscore - minscore) + yoffset;
   return(y);
@@ -1190,7 +1196,7 @@ scale_hypo_y(LOGPROB s)
 /* draw popped words */
 /** 
  * <JA>
- * åФ줿褹롥
+ * åФ줿褹. 
  * 
  * @param widget [in] 襦å
  * @param p [in] åФ줿ξ
@@ -1255,9 +1261,9 @@ draw_popped(GtkWidget *widget, POPNODE *p, UserColors c, int width, int style)
 		       5,
 		       5);
     if (p->tre->wid != WORD_INVALID) {
-      text_width = gdk_string_width(fontset, winfo->woutput[p->tre->wid]) + 1;
+      text_width = gdk_string_width(fontset, re->model->winfo->woutput[p->tre->wid]) + 1;
       gdk_draw_string(pixmap, fontset, dgc, x - text_width-1, y - 5, 
-		      winfo->woutput[p->tre->wid]);
+		      re->model->winfo->woutput[p->tre->wid]);
     }
   }
     
@@ -1270,7 +1276,7 @@ static int old_popctr;		///< popctr of previously popped hypo.
 
 /** 
  * <JA>
- * 2ѥñŸͻҤ褹롥
+ * 2ѥñŸͻҤ褹. 
  * 
  * @param widget [in] 襦å
  * @param popctr [in] Ÿֹ
@@ -1288,7 +1294,7 @@ draw_popnodes(GtkWidget *widget, int popctr)
   POPNODE *p, *porg;
 
   if (popctr < 0 || popctr >= pnum) {
-    j_printerr("invalid popctr (%d > %d)!\n", popctr, pnum);
+    fprintf(stderr, "invalid popctr (%d > %d)!\n", popctr, pnum);
     return;
   }
   
@@ -1309,7 +1315,7 @@ draw_popnodes(GtkWidget *widget, int popctr)
 }
 /** 
  * <JA>
- * ľ draw_popnodes() 褷Τ񤭤ƾä
+ * ľ draw_popnodes() 褷Τ񤭤ƾä. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -1386,7 +1392,7 @@ draw_background(GtkWidget *widget)
 
 /** 
  * <JA>
- * 襭ѥɽ٤Ƥñ pixmap 褹롥
+ * 襭ѥɽ٤Ƥñ pixmap 褹. 
  * 
  * @param widget 
  * </JA>
@@ -1408,7 +1414,7 @@ drawarea_draw(GtkWidget *widget)
   /* make background */
   draw_background(widget);
 
-  if (speechlen != 0) {
+  if (re->speechlen != 0) {
     draw_waveform(widget);
   }
 
@@ -1432,7 +1438,7 @@ drawarea_draw(GtkWidget *widget)
 
 /** 
  * <JA>
- * Expose ٥ȤȯԤ롥
+ * Expose ٥ȤȯԤ. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -1456,13 +1462,13 @@ drawarea_expose(GtkWidget *widget)
 
 /** 
  * <JA>
- * Expose ٥Ƚ餫褵줿 pixmap ̤ɽ롥
+ * Expose ٥Ƚ餫褵줿 pixmap ̤ɽ. 
  * 
  * @param widget [in] 襦å
  * @param event [in] ٥Ⱦ
  * @param user_data [in] 桼ǡ̤ѡ
  * 
- * @return gboolean ֤
+ * @return gboolean ֤. 
  * </JA>
  * <EN>
  * Expose event handler: show the already drawn pixmap to the screen.
@@ -1489,13 +1495,13 @@ event_drawarea_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_da
 
 /** 
  * <JA>
- * Configure ٥Ƚresize ޤ map ˺褹롥
+ * Configure ٥Ƚresize ޤ map ˺褹. 
  * 
  * @param widget [in] 襦å
  * @param event [in] ٥Ⱦ
  * @param user_data [in] 桼ǡ̤ѡ
  * 
- * @return gboolean ֤
+ * @return gboolean ֤. 
  * </JA>
  * <EN>
  * Configure event handler: redraw objects when resized or mapped.
@@ -1528,7 +1534,7 @@ event_drawarea_configure(GtkWidget *widget, GdkEventExpose *event, gpointer user
 /** 
  * <JA>
  * [show threshold] ܥ󥯥å callback: ȥꥬ
- *  ON/ OFF
+ *  ON/ OFF. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -1543,7 +1549,7 @@ static void
 action_toggle_thres(GtkWidget *widget)
 {
 
-  if (speechlen == 0) return;
+  if (re->speechlen == 0) return;
   /* toggle switch */
   if (sw_level_thres) sw_level_thres = FALSE;
   else sw_level_thres = TRUE;
@@ -1558,7 +1564,7 @@ action_toggle_thres(GtkWidget *widget)
 #ifdef PLAYCOMMAND
 /** 
  * <JA>
- * [play] ܥ󥯥å callback: 롥
+ * [play] ܥ󥯥å callback: . 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -1575,22 +1581,22 @@ action_play_waveform(GtkWidget *widget)
   static char command[250];
   int fd;
 
-  if (speechlen == 0) return;
+  if (re->speechlen == 0) return;
   
   /* play waveform */
   snprintf(buf, 250, "/var/tmp/julius_visual_play.%d", getpid());
   if ((fd = open(buf, O_CREAT | O_TRUNC | O_WRONLY, 0644)) < 0) {
-    j_printerr("cannot open %s for writing\n", buf);
+    fprintf(stderr, "cannot open %s for writing\n", buf);
     return;
   }
-  if (wrsamp(fd, speech, speechlen) < 0) {
-    j_printerr("failed to write to %s for playing\n", buf);
+  if (wrsamp(fd, re->speech, re->speechlen) < 0) {
+    fprintf(stderr, "failed to write to %s for playing\n", buf);
     return;
   }
   close(fd);
   
-  snprintf(command, 250, PLAYCOMMAND, smpFreq, buf);
-  j_printf("play: [%s]\n", command);
+  snprintf(command, 250, PLAYCOMMAND, re->jconf->analysis.para.smp_freq, buf);
+  printf("play: [%s]\n", command);
   system(command);
 
   unlink(buf);
@@ -1599,7 +1605,7 @@ action_play_waveform(GtkWidget *widget)
 
 /** 
  * <JA>
- * [Word view] ܥ󥯥å callback: YñIDˤ롥
+ * [Word view] ܥ󥯥å callback: YñIDˤ. 
  * 
  * @param button [in] ܥ󥦥å
  * @param widget [in] 襦å
@@ -1629,7 +1635,7 @@ action_view_wid(GtkWidget *button, GtkWidget *widget)
 
 /** 
  * <JA>
- * [Score view] ܥ󥯥å callback: Y򥹥ˤ롥
+ * [Score view] ܥ󥯥å callback: Y򥹥ˤ. 
  * 
  * @param button [in] ܥ󥦥å
  * @param widget [in] 襦å
@@ -1658,7 +1664,7 @@ action_view_score(GtkWidget *button, GtkWidget *widget)
 /** 
  * <JA>
  * [Beam view] ܥ󥯥å callback: Yե졼ऴȤκ祹
- * κʬˤ롥
+ * κʬˤ. 
  * 
  * @param button [in] ܥ󥦥å
  * @param widget [in] 襦å
@@ -1688,7 +1694,7 @@ action_view_beam(GtkWidget *button, GtkWidget *widget)
 /** 
  * <JA>
  * [Arc on/off] ܥ󥯥å callback: ñƬñ콪üδ֤
- *  on/off 롥
+ *  on/off . 
  * 
  * @param button [in] ܥ󥦥å
  * @param widget [in] 襦å
@@ -1720,7 +1726,7 @@ action_toggle_arc(GtkWidget *button, GtkWidget *widget)
 /** 
  * <JA>
  * ƥȥåȤñ̾Ϥ줿Ȥcallback: ñꥹȤ
- * 롥
+ * . 
  * 
  * @param widget [in] ƥȥå
  * @param draw [in] 襦å
@@ -1743,7 +1749,7 @@ action_set_wid(GtkWidget *widget, GtkWidget *draw)
 
   /* allocate */
   if (wordlist == NULL) {
-    wordlist = mymalloc(sizeof(WORD_ID) * winfo->num);
+    wordlist = mymalloc(sizeof(WORD_ID) * re->model->winfo->num);
   }
   wordlistnum = 0;
 
@@ -1751,16 +1757,16 @@ action_set_wid(GtkWidget *widget, GtkWidget *draw)
   if (strlen(entry_text) == 0) {
     wordlistnum = 0;
   } else {
-    for (i=0;i<winfo->num;i++) {
-      if (strmatch(entry_text, winfo->woutput[i])) {
+    for (i=0;i<re->model->winfo->num;i++) {
+      if (strmatch(entry_text, re->model->winfo->woutput[i])) {
 	wordlist[wordlistnum] = i;
 	wordlistnum++;
       }
     }
     if (wordlistnum == 0) {
-      j_printerr("word \"%s\" not found, show all\n", entry_text);
+      fprintf(stderr, "word \"%s\" not found, show all\n", entry_text);
     } else {
-      j_printerr("%d words found for \"%s\"\n", wordlistnum, entry_text);
+      fprintf(stderr, "%d words found for \"%s\"\n", wordlistnum, entry_text);
     }
   }
   
@@ -1772,8 +1778,8 @@ action_set_wid(GtkWidget *widget, GtkWidget *draw)
 
 /** 
  * <JA>
- * [x2] ܥ󥯥åcallback: X2ܤ˿ĥ롥ʤFITSCREEN
- * OFFˤʤ롥
+ * [x2] ܥ󥯥åcallback: X2ܤ˿ĥ. ʤFITSCREEN
+ * OFFˤʤ. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -1799,8 +1805,8 @@ action_zoom(GtkWidget *widget)
 
 /** 
  * <JA>
- * [x4] ܥ󥯥åcallback: X4ܤ˿ĥ롥ʤFITSCREEN
- * OFFˤʤ롥
+ * [x4] ܥ󥯥åcallback: X4ܤ˿ĥ. ʤFITSCREEN
+ * OFFˤʤ. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -1826,8 +1832,8 @@ action_zoom_4(GtkWidget *widget)
 
 /** 
  * <JA>
- * [x8] ܥ󥯥åcallback: X8ܤ˿ĥ롥ʤFITSCREEN
- * OFFˤʤ롥
+ * [x8] ܥ󥯥åcallback: X8ܤ˿ĥ. ʤFITSCREEN
+ * OFFˤʤ. 
  * 
  * @param widget [in] 襦å
  * </JA>
@@ -1854,7 +1860,7 @@ action_zoom_8(GtkWidget *widget)
 /** 
  * <JA>
  * [Fit] ܥ󥯥åΥХåѥ򥦥ɥ
- * ưŪ˹碌褦ˤ롥¾ zoom  off ˤ롥
+ * ưŪ˹碌褦ˤ. ¾ zoom  off ˤ. 
  * 
  * @param widget 
  * </JA>
@@ -1878,7 +1884,7 @@ action_fit_screen(GtkWidget *widget)
 
 /** 
  * <JA>
- * 2ѥƸѤ [start] ܥcallback: 2ѥƸѤ˽롥
+ * 2ѥƸѤ [start] ܥcallback: 2ѥƸѤ˽. 
  * 
  * @param button [in] ܥ󥦥å
  * @param widget [in] 襦å
@@ -1906,7 +1912,7 @@ action_toggle_popctr(GtkWidget *button, GtkWidget *widget)
 /** 
  * <JA>
  * 2ѥƸѤΥcallback: ͤ N ѹ줿Ȥˡ
- * N ܤñŸͻҤ褹롥
+ * N ܤñŸͻҤ褹. 
  * 
  * @param adj [in] 㥹
  * @param widget [in] 襦å
@@ -1945,7 +1951,7 @@ action_change_popctr(GtkAdjustment *adj, GtkWidget *widget)
  * @param event [in] ٥
  * @param data [in] 桼ǡ
  * 
- * @return FALSE֤destroy signalȯԤ뤿ˡ
+ * @return FALSE֤destroy signalȯԤ뤿. 
  * </JA>
  * <EN>
  * GTK delete event handler.
@@ -1965,7 +1971,7 @@ delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 
 /** 
  * <JA>
- * GTKνλ٥ȥϥɥ顥ץꥱλ롥
+ * GTKνλ٥ȥϥɥ. ץꥱλ. 
  * 
  * @param widget [in] å
  * @param data [in] 桼ǡ
@@ -1989,17 +1995,20 @@ destroy(GtkWidget *widget, gpointer data)
 
 /** 
  * <JA>
- * ưĻ벽ǽ롥
+ * ưĻ벽ǽ. 
  * </JA>
  * <EN>
  * Initialize visualization functions at startup.
  * </EN>
  */
 void
-visual_init()
+visual_init(Recog *recog)
 {
   POPNODE *p;
 
+  /* hold recognition instance to local */
+  re = recog;
+
   /* reset values */
   btlocal = NULL;
   
@@ -2014,19 +2023,19 @@ visual_init()
   /* load fontset */
   fontset = gdk_fontset_load(FONTSET);
   if (fontset == NULL) {
-    j_error("cannot load X font \"%s\" for visualize\n", FONTSET);
+    fprintf(stderr, "cannot load X font \"%s\" for visualize\n", FONTSET); exit(-1);
   }
 
   /* initialize color */
   color_init();
 
-  j_printerr("GTK initialized\n");
+  fprintf(stderr, "GTK initialized\n");
 
 }
 
 /** 
  * <JA>
- * ǧ̤򸵤ˡõ֤βĻ벽¹Ԥ롥
+ * ǧ̤򸵤ˡõ֤βĻ벽¹Ԥ. 
  * 
  * @param bt [in] ñȥꥹ
  * </JA>
@@ -2046,7 +2055,7 @@ visual_show(BACKTRELLIS *bt)
   GList *glist;
 
   
-  j_printerr("*** Showing word trellis view (close window to proceed)\n");
+  fprintf(stderr, "*** Showing word trellis view (close window to proceed)\n");
 
   /* store pointer to backtrellis data */
   btlocal = bt;
@@ -2058,7 +2067,7 @@ visual_show(BACKTRELLIS *bt)
   get_max_hypo_score();
 
   /* prepare for waveform */
-  if (speechlen != 0) get_max_waveform_level();
+  if (re->speechlen != 0) get_max_waveform_level();
 
   /* start with trellis view */
   sw_hypo = FALSE;
@@ -2100,7 +2109,7 @@ visual_show(BACKTRELLIS *bt)
   box2 = gtk_vbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(box1), box2, FALSE, TRUE, 0);
 
-  if (speechlen != 0) {
+  if (re->speechlen != 0) {
     /* create waveform related frame */
     frame = gtk_frame_new("waveform");
     gtk_box_pack_start(GTK_BOX(box2), frame, FALSE, FALSE, 0);
