@@ -17,13 +17,13 @@
  * @author Akinobu Lee
  * @date   Fri Feb 16 15:05:43 2007
  *
- * $Revision: 1.8 $
+ * $Revision: 1.14 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -56,7 +56,7 @@
 void
 jconf_set_default_values(Jconf *j)
 {
-  j->input.type				= INPUT_WAVEFORM;
+  j->input.type				= INPUT_VECTOR;
   j->input.speech_input			= SP_MFCFILE;
   j->input.device			= SP_INPUT_DEFAULT;
   j->input.plugin_source		= -1;
@@ -77,6 +77,7 @@ jconf_set_default_values(Jconf *j)
   j->detect.tail_margin_msec		= 400;
   j->detect.zero_cross_num		= 60;
   j->detect.silence_cut			= 2; /* accept device default */
+  j->detect.chunk_size			= 1000;
 #ifdef GMM_VAD
   j->detect.gmm_margin			= DEFAULT_GMM_MARGIN;
   j->detect.gmm_uptrigger_thres		= 0.7;
@@ -192,7 +193,10 @@ jconf_set_default_values_lm(JCONF_LM *j)
 #endif
   strcpy(j->wordrecog_head_silence_model_name, "silB");
   strcpy(j->wordrecog_tail_silence_model_name, "silE");
-  j->wordrecog_silence_context_name[0]	= '\0';
+  j->wordrecog_silence_context_name[0] = '\0';
+  strcpy(j->unknown_name, UNK_WORD_DEFAULT); // or UNK_WORD_DEFAULT2
+  j->additional_dict_files		= NULL;
+  j->additional_dict_entries		= NULL;
 }
 
 /** 
@@ -232,6 +236,9 @@ jconf_set_default_values_search(JCONF_SEARCH *j)
   j->lmp.lmp_specified			= FALSE;
 
   j->pass1.specified_trellis_beam_width	= -1;
+#ifdef SCORE_PRUNING
+  j->pass1.score_pruning_width		= -1.0;
+#endif
 #if defined(WPAIR) && defined(WPAIR_KEEP_NLIMIT)
   j->pass1.wpair_keep_nlimit		= 3;
 #endif
