@@ -12,13 +12,13 @@
  * @author Akinobu Lee
  * @date   Sun Oct 28 18:06:20 2007
  *
- * $Revision: 1.4 $
+ * $Revision: 1.7 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -160,11 +160,11 @@ j_process_am_new(Recog *recog, JCONF_AM *amconf)
 void
 j_process_am_free(PROCESS_AM *am)
 {
+  /* HMMWork hmmwrk */
+  outprob_free(&(am->hmmwrk));
   if (am->hmminfo) hmminfo_free(am->hmminfo);
   if (am->hmm_gs) hmminfo_free(am->hmm_gs);
   /* not free am->jconf  */
-  /* HMMWork hmmwrk */
-  outprob_free(&(am->hmmwrk));
   free(am);
 }
 
@@ -300,6 +300,10 @@ j_recogprocess_free(RecogProcess *process)
   /* not free jconf, am, lm here */
   /* free part of StackDecode work area */
   wchmm_fbs_free(process);
+  /* free cache */
+  if (process->lmtype == LM_PROB) {
+    max_successor_cache_free(process->wchmm);
+  }
   /* free wchmm */
   if (process->wchmm) wchmm_free(process->wchmm);
   /* free backtrellis */
@@ -457,6 +461,21 @@ j_jconf_lm_new()
 void
 j_jconf_lm_free(JCONF_LM *lmconf)
 {
+  JCONF_LM_NAMELIST *nl, *nltmp;
+  nl = lmconf->additional_dict_files;
+  while (nl) {
+    nltmp = nl->next;
+    free(nl->name);
+    free(nl);
+    nl = nltmp;
+  }
+  nl = lmconf->additional_dict_entries;
+  while (nl) {
+    nltmp = nl->next;
+    free(nl->name);
+    free(nl);
+    nl = nltmp;
+  }
   free(lmconf);
 }
 
