@@ -18,13 +18,13 @@
  * @author Akinobu Lee
  * @date   Thu May 12 18:52:07 2005
  *
- * $Revision: 1.15 $
+ * $Revision: 1.25 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -283,6 +283,16 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
 	if ((lmconf = j_get_lmconf_by_name(jconf, tmparg)) == NULL) return FALSE;
       }
 
+      /* check to avoid assigning an LM for multiple SR */
+      for(sconf=jconf->search_root;sconf;sconf=sconf->next) {
+	if (sconf->lmconf == lmconf) {
+	  jlog("ERROR: you are going to share LM \"%s\" among multiple SRs\n");
+	  jlog("ERROR: current Julius cannot share LM among SRs\n");
+	  jlog("ERROR: you should define LM for each SR\n");
+	  return FALSE;
+	}
+      }
+
       /* if not first time, create new module instance and switch to it */
       sconf = j_jconf_search_new();
       sconf->amconf = amconf;
@@ -364,7 +374,17 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
 	jconf->input.device = SP_INPUT_ESD;
 	jconf->decodeopt.realtime_flag = TRUE;
 #else
-	jlog("ERROR: m_options: \"-input oss\": OSS support is not built-in\n");
+	jlog("ERROR: m_options: \"-input esd\": ESounD support is not built-in\n");
+	return FALSE;
+#endif
+      } else if (strmatch(tmparg,"pulseaudio")) {
+#ifdef HAS_PULSEAUDIO
+	jconf->input.type = INPUT_WAVEFORM;
+	jconf->input.speech_input = SP_MIC;
+	jconf->input.device = SP_INPUT_PULSEAUDIO;
+	jconf->decodeopt.realtime_flag = TRUE;
+#else
+	jlog("ERROR: m_options: \"-input pulseaudio\": PulseAudio support is not built-in\n");
 	return FALSE;
 #endif
 #endif
@@ -549,6 +569,13 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
       jlog("WARNING: m_options: SCAN_BEAM disabled, \"-sb\" ignored\n");
 #endif
       continue;
+#ifdef SCORE_PRUNING
+    } else if (strmatch(argv[i],"-bs")) { /* score beam width for 1st pass */
+      if (!check_section(jconf, argv[i], JCONF_OPT_SR)) return FALSE;
+      GET_TMPARG;
+      jconf->searchnow->pass1.score_pruning_width = atof(tmparg);
+      continue;
+#endif
     } else if (strmatch(argv[i],"-discount")) {	/* (bogus) */
       jlog("WARNING: m_options: option \"-discount\" is now bogus, ignored\n");
       continue;
@@ -588,6 +615,11 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
       GET_TMPARG;
       jconf->detect.tail_margin_msec = atoi(tmparg);
       continue;
+    } else if (strmatch(argv[i],"-chunksize")) { /* chunk size for detection */
+      if (!check_section(jconf, argv[i], JCONF_OPT_GLOBAL)) return FALSE; 
+      GET_TMPARG;
+      jconf->detect.chunk_size = atoi(tmparg);
+      continue;
     } else if (strmatch(argv[i],"-hipass")||strmatch(argv[i],"-hifreq")) { /* frequency of upper band limit */
       if (!check_section(jconf, argv[i], JCONF_OPT_AM)) return FALSE; 
       GET_TMPARG;
@@ -886,6 +918,11 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
       GET_TMPARG;
       jconf->lmnow->tail_silname = strcpy((char*)mymalloc(strlen(tmparg)+1),tmparg);
       continue;
+    } else if (strmatch(argv[i],"-mapunk")) { /* unknown word */
+      if (!check_section(jconf, argv[i], JCONF_OPT_LM)) return FALSE; 
+      GET_TMPARG;
+      strncpy(jconf->lmnow->unknown_name, tmparg, UNK_WORD_MAXLEN);
+      continue;
     } else if (strmatch(argv[i],"-iwspword")) { /* add short pause word */
       if (!check_section(jconf, argv[i], JCONF_OPT_LM)) return FALSE; 
       jconf->lmnow->enable_iwspword = TRUE;
@@ -1186,10 +1223,13 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
     } else if (strmatch(argv[i],"-htkconf")) {
       if (!check_section(jconf, argv[i], JCONF_OPT_AM)) return FALSE; 
       GET_TMPARG;
+      tmparg = filepath(tmparg, cwd);
       if (htk_config_file_parse(tmparg, &(jconf->amnow->analysis.para_htk)) == FALSE) {
 	jlog("ERROR: m_options: failed to read %s\n", tmparg);
+	free(tmparg);
 	return FALSE;
       }
+      free(tmparg);
       continue;
     } else if (strmatch(argv[i], "-wlist")) {
       if (!check_section(jconf, argv[i], JCONF_OPT_LM)) return FALSE; 
@@ -1252,6 +1292,18 @@ opt_parse(int argc, char *argv[], char *cwd, Jconf *jconf)
       plugin_load_dirs(tmparg);
       continue;
 #endif
+    } else if (strmatch(argv[i],"-adddict")) {
+	if (!check_section(jconf, argv[i], JCONF_OPT_LM)) return FALSE; 
+	GET_TMPARG;
+	tmparg = filepath(tmparg, cwd);
+	j_add_dict(jconf->lmnow, tmparg);
+	free(tmparg);
+	continue;
+    } else if (strmatch(argv[i],"-addentry")) {
+	if (!check_section(jconf, argv[i], JCONF_OPT_LM)) return FALSE; 
+	GET_TMPARG;
+	j_add_word(jconf->lmnow, tmparg);
+	continue;
     }
     if (argv[i][0] == '-' && strlen(argv[i]) == 2) {
       /* 1-letter options */
