@@ -95,13 +95,13 @@
  * @author Akinobu LEE
  * @date   Fri Feb 11 15:04:02 2005
  *
- * $Revision: 1.8 $
+ * $Revision: 1.11 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -182,6 +182,9 @@ typedef struct __ngram_info__ {
   LOGPROB *bo_wt_1;		///< back-off weights for 2-gram on 1st pass
   LOGPROB *p_2;			///< 2-gram prob for the 1st pass
   LOGPROB (*bigram_prob)(struct __ngram_info__ *, WORD_ID, WORD_ID); ///< Pointer of a function to compite bigram probability on the 1st pass.  See bi_prob_func_set() for details
+
+  BMALLOC_BASE *mroot;		///< Pointer for block memory allocation for lookup index
+
 } NGRAM_INFO;
 
 
@@ -242,6 +245,7 @@ void ngram_info_free(NGRAM_INFO *ngram);
 boolean init_ngram_bin(NGRAM_INFO *ndata, char *ngram_file);
 boolean init_ngram_arpa(NGRAM_INFO *ndata, char *ngram_file, int dir);
 boolean init_ngram_arpa_additional(NGRAM_INFO *ndata, char *bigram_file);
+void set_default_unknown_id(NGRAM_INFO *ndata);
 void set_unknown_id(NGRAM_INFO *ndata, char *str);
 
 void print_ngram_info(FILE *fp, NGRAM_INFO *ndata);
