@@ -5,28 +5,31 @@
  * @brief  ޥ (Linux) - ǥեȥǥХ
  *
  * ޥϤΤ٥ؿǤ
- * 󥿥եꤷʤ˸ƤФޤ
- * ºݤˤϡALSA, OSS, ESD νǺǽ˸ĤäΤѤޤ
+ * 󥿥եꤷʤ (-input mic) ˸ƤФޤ
+ * ALSA, PulesAudio, OSS, ESD νǺǽ˸ĤäΤѤޤ
+ * 줾 API Ū˻ꤷ "-input" ˤ줾
+ * "alsa", "oss", "pulseaudio", "esd" ꤷƤ
  * </JA>
  * <EN>
  * @brief  Microphone input on Linux - default device
  *
  * Low level I/O functions for microphone input on Linux.
- * This will be called when no device was explicitly specified.
- * A call to the functions in this file will be redirected to
- * ALSA, OSS or ESD.
+ * This will be called when no device was explicitly specified ("-input mic").
+ * ALSA, PulseAudio, OSS, ESD will be chosen in this order at compilation time.
+ * "-input alsa", "-input oss", "-input pulseaudio" or "-input esd" to
+ * specify which API to use.
  * </EN>
  *
  * @author Akinobu LEE
  * @date   Sun Feb 13 16:18:26 2005
  *
- * $Revision: 1.4 $
+ * $Revision: 1.7 $
  * 
  */
 /*
- * Copyright (c) 1991-2008 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2008 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -48,10 +51,12 @@ adin_mic_standby(int sfreq, void *dummy)
   return(adin_alsa_standby(sfreq, dummy));
 #elif defined(HAS_OSS)
   return(adin_oss_standby(sfreq, dummy));
+#elif defined(HAS_PULSEAUDIO)
+  return(adin_pulseaudio_standby(sfreq, dummy));
 #elif defined(HAS_ESD)
   return(adin_esd_standby(sfreq, dummy));
 #else  /* other than Linux */
-  jlog("Error: neither of alsa/oss/esd device is available\n");
+  jlog("Error: neither of pulseaudio/alsa/oss/esd device is available\n");
   return FALSE;
 #endif
 }
@@ -70,10 +75,12 @@ adin_mic_begin(char *pathname)
   return(adin_alsa_begin(pathname));
 #elif defined(HAS_OSS)
   return(adin_oss_begin(pathname));
+#elif defined(HAS_PULSEAUDIO)
+  return(adin_pulseaudio_begin(pathname));
 #elif defined(HAS_ESD)
   return(adin_esd_begin(pathname));
 #else  /* other than Linux */
-  jlog("Error: neither of alsa/oss/esd device is available\n");
+  jlog("Error: neither of pulseaudio/alsa/oss/esd device is available\n");
   return FALSE;
 #endif
 }
@@ -90,10 +97,12 @@ adin_mic_end()
   return(adin_alsa_end());
 #elif defined(HAS_OSS)
   return(adin_oss_end());
+#elif defined(HAS_PULSEAUDIO)
+  return(adin_pulseaudio_end());
 #elif defined(HAS_ESD)
   return(adin_esd_end());
 #else  /* other than Linux */
-  jlog("Error: neither of alsa/oss/esd device is available\n");
+  jlog("Error: neither of pulseaudio/alsa/oss/esd device is available\n");
   return FALSE;
 #endif
 }
@@ -117,10 +126,12 @@ adin_mic_read(SP16 *buf, int sampnum)
   return(adin_alsa_read(buf, sampnum));
 #elif defined(HAS_OSS)
   return(adin_oss_read(buf, sampnum));
+#elif defined(HAS_PULSEAUDIO)
+  return(adin_pulseaudio_read(buf, sampnum));
 #elif defined(HAS_ESD)
   return(adin_esd_read(buf, sampnum));
 #else  /* other than Linux */
-  jlog("Error: neither of alsa/oss/esd device is available\n");
+  jlog("Error: neither of pulseaudio/alsa/oss/esd device is available\n");
   return -2;
 #endif
 }
@@ -139,9 +150,11 @@ adin_mic_input_name()
   return(adin_alsa_input_name());
 #elif defined(HAS_OSS)
   return(adin_oss_input_name());
+#elif defined(HAS_PULSEAUDIO)
+  return(adin_pulseaudio_input_name());
 #elif defined(HAS_ESD)
   return(adin_esd_input_name());
 #else  /* other than Linux */
-  return("Error: neither of alsa/oss/esd device is available\n");
+  return("Error: neither of pulseaudio/alsa/oss/esd device is available\n");
 #endif
 }
