@@ -26,13 +26,13 @@
  * @author Akinobu LEE
  * @date   Sun Feb 13 19:50:55 2005
  *
- * $Revision: 1.2 $
+ * $Revision: 1.5 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -46,6 +46,8 @@
 #include <sent/stddefs.h>
 #include <sent/adin.h>
 
+static char server_devname[MAXPATHLEN];  ///< Device name
+
 /** 
  * Connection initialization: check connectivity and open for recording.
  * 
@@ -57,8 +59,7 @@
 boolean
 adin_netaudio_standby(int sfreq, void *arg)
 {
-  char *server_devname;
-  server_devname = arg;
+  strncpy(server_devname, arg, MAXPATHLEN);
   if (NA_standby(sfreq, server_devname) == 0) return(FALSE); /* error */
   return(TRUE);
 }
@@ -66,10 +67,12 @@ adin_netaudio_standby(int sfreq, void *arg)
 /** 
  * Start recording.
  * 
+ * @param pathname [in] path name to open or NULL for default
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_netaudio_begin()
+adin_netaudio_begin(char *pathname)
 {
   NA_start();
   return(TRUE);
@@ -110,3 +113,16 @@ adin_netaudio_read(SP16 *buf, int sampnum)
   }
   return(cnt);
 }
+
+/** 
+ * 
+ * Function to return current input source device name
+ * 
+ * @return string of current input device name.
+ * 
+ */
+char *
+adin_netaudio_input_name()
+{
+  return(server_devname);
+}
