@@ -56,12 +56,12 @@
  * @author Akinobu LEE
  * @date   Mon Feb 14 12:03:48 2005
  *
- * $Revision: 1.12 $
+ * $Revision: 1.18 $
  * 
  */
 /*
  * Copyright (c) 2004-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -193,9 +193,155 @@ Callback(const void *inbuf, void *outbuf, unsigned long len, const PaStreamCallb
 }
 
 #ifdef OLDVER
-static PortAudioStream *stream;		///< Stream information
+static PortAudioStream *stream = NULL;		///< Stream information
 #else
-static PaStream *stream;		///< Stream information
+static PaStream *stream = NULL;		///< Stream information
+#endif
+static int srate;		///< Required sampling rate
+
+
+#ifdef CHOOSE_HOST_API
+
+// Get device list
+// If first argument is NULL, return the maximum number of devices
+int
+get_device_list(int *devidlist, char **namelist, int maxstrlen, int maxnum)
+{
+  PaDeviceIndex numDevice = Pa_GetDeviceCount(), i;
+  const PaDeviceInfo *deviceInfo;
+  const PaHostApiInfo *apiInfo;
+  static char buf[256];
+  int n;
+
+  n = 0;
+  for(i=0;i<numDevice;i++) {
+    deviceInfo = Pa_GetDeviceInfo(i);
+    if (!deviceInfo) continue;
+    if (deviceInfo->maxInputChannels <= 0) continue;
+    apiInfo = Pa_GetHostApiInfo(deviceInfo->hostApi);
+    if (!apiInfo) continue;
+    if (devidlist != NULL) {
+      if ( n >= maxnum ) break;
+      snprintf(buf, 255, "%s: %s", apiInfo->name, deviceInfo->name);
+      buf[255] = '\0';
+      devidlist[n] = i;
+      strncpy(namelist[n], buf, maxstrlen);
+    }
+    n++;
+  }
+  
+  return n;
+}
+
+// Automatically choose a device to open
+// 
+// 1. if the env value of "PORTAUDIO_DEV" is defined and matches any of 
+//    "apiInfo->name: deviceInfo->name" string, use it.
+// 2. if the env value "PORTAUDIO_DEV_NUM" is defined, use the (value-1)
+//    as device id.
+// 3. if not yet, default device will be chosen:
+// 3.1. in WIN32 environment, search for supported and available API in
+//      the following order: WASAPI, ASIO, DirectSound, MME.
+//      Note that only the APIs supported by the linked PortAudio
+//      library are available.
+// 3.2  in other environment, use the default input device.
+//
+// store the device id to devId_ret and returns 0.
+// if devId_ret is -1, tell caller to use default.
+// returns -1 on error.
+static int
+auto_determine_device(int *devId_ret)
+{
+  int devId;
+  PaDeviceIndex numDevice = Pa_GetDeviceCount(), i;
+  const PaDeviceInfo *deviceInfo;
+  const PaHostApiInfo *apiInfo;
+  char *devname;
+  static char buf[256];
+#if defined(_WIN32) || defined(__CYGWIN__)
+  // at win32, force preference order: iWASAPI > ASIO > DirectSound > MME > Other
+  int iMME = -1, iDS = -1, iASIO = -1, iWASAPI = -1;
+#endif
+
+  // if PORTAUDIO_DEV is specified, match it against available APIs
+  devname = getenv("PORTAUDIO_DEV");
+  devId = -1;
+
+  // get list of available capture devices
+  jlog("Stat: adin_portaudio: sound capture devices:\n");
+  for(i=0;i<numDevice;i++) {
+    deviceInfo = Pa_GetDeviceInfo(i);
+    if (!deviceInfo) continue;
+    if (deviceInfo->maxInputChannels <= 0) continue;
+    apiInfo = Pa_GetHostApiInfo(deviceInfo->hostApi);
+    if (!apiInfo) continue;
+    snprintf(buf, 255, "%s: %s", apiInfo->name, deviceInfo->name);
+    buf[255] = '\0';
+    jlog("  %d [%s]\n", i+1, buf);
+    if (devname && !strncmp(devname, buf, strlen(devname))) {
+      // device name (partially) matches PORTAUDIO_DEV
+      devId = i;
+    }
+#if defined(_WIN32) || defined(__CYGWIN__)
+    // store the device ID for each API
+    switch(apiInfo->type) {
+    case paWASAPI: if (iWASAPI < 0) iWASAPI = i; break;
+    case paMME:if (iMME   < 0) iMME   = i; break;
+    case paDirectSound:if (iDS    < 0) iDS    = i; break;
+    case paASIO:if (iASIO  < 0) iASIO  = i; break;
+    }
+#endif
+  }
+  if (devname) {
+    if (devId == -1) {
+      jlog("Error: adin_portaudio: PORTAUDIO_DEV=\"%s\", but no device matches it\n", devname);
+      return -1;
+    }
+    jlog("  --> #%d matches PORTAUDIO_DEV, use it\n", devId + 1);
+    *devId_ret = devId;
+    return 0;
+  }
+  if (getenv("PORTAUDIO_DEV_NUM")) {
+    devId = atoi(getenv("PORTAUDIO_DEV_NUM")) - 1;
+    if (devId < 0 || devId >= numDevice) {
+      jlog("Error: PORTAUDIO_DEV_NUM=%d, but device %d not exist\n", devId+1, devId+1);
+      return -1;
+    }
+    jlog("  --> use device %d, specified by PORTAUDIO_DEV_NUM\n", devId + 1);
+    *devId_ret = devId;
+    return 0;
+  }
+#if defined(_WIN32) || defined(__CYGWIN__)
+  jlog("Stat: adin_portaudio: APIs:");
+  if (iWASAPI >= 0) jlog(" WASAPI");
+  if (iASIO >= 0) jlog(" ASIO");
+  if (iDS >= 0) jlog(" DirectSound");
+  if (iMME >= 0) jlog(" MME");
+  jlog("\n");
+  if (iWASAPI >= 0) {
+    jlog("Stat: adin_portaudio: -- WASAPI selected\n");
+    devId = iWASAPI;
+  } else if (iASIO >= 0) {
+    jlog("Stat: adin_portaudio: -- ASIO selected\n");
+    devId = iASIO;
+  } else if (iDS >= 0) {
+    jlog("Stat: adin_portaudio: -- DirectSound selected\n");
+    devId = iDS;
+  } else if (iMME >= 0) {
+    jlog("Stat: adin_portaudio: -- MME selected\n");
+    devId = iMME;
+  } else {
+    jlog("Error: adin_portaudio: no device available, try default\n");
+    devId = -1;
+  }
+  *devId_ret = devId;
+#else
+  jlog("Stat: adin_portaudio: use default device\n");
+  *devId_ret = -1;
+#endif
+  return 0;
+}
+
 #endif
 
 /** 
@@ -209,6 +355,22 @@ static PaStream *stream;		///< Stream information
 boolean
 adin_mic_standby(int sfreq, void *dummy)
 {
+  /* store required sampling rate for checking after opening device */
+  srate = sfreq;
+  return TRUE;
+}
+
+/** 
+ * Open the portaudio device and check capability of the opening device.
+ *
+ * @param arg [in] argument: if number, use it as device ID
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
+static boolean
+adin_mic_open(char *arg)
+{
+  int sfreq = srate;
   PaError err;
 #ifdef OLDVER
   int frames_per_buffer;
@@ -216,10 +378,11 @@ adin_mic_standby(int sfreq, void *dummy)
 #endif
   int latency;
   char *p;
+  int devId;
 
   /* set cycle buffer length */
   cycle_buffer_len = INPUT_DELAY_SEC * sfreq;
-  jlog("Stat: adin_portaudio: INPUT_DELAY_SEC = %d\n", INPUT_DELAY_SEC);
+  //jlog("Stat: adin_portaudio: INPUT_DELAY_SEC = %d\n", INPUT_DELAY_SEC);
   jlog("Stat: adin_portaudio: audio cycle buffer length = %d bytes\n", cycle_buffer_len * sizeof(SP16));
 
 #ifdef OLDVER
@@ -266,114 +429,76 @@ adin_mic_standby(int sfreq, void *dummy)
 
 #ifdef CHOOSE_HOST_API
 
+  if (arg == NULL) {
+    // first try to determine the best device
+    if (auto_determine_device(&devId) == -1) {
+      jlog("Error: adin_portaudio: failed to choose the specified device\n");
+      return(FALSE);
+    }
+    if (devId == -1) {
+      // No device has been determined, use the default input device
+      devId = Pa_GetDefaultInputDevice();
+      if (devId == paNoDevice) {
+	jlog("Error: adin_portaudio: no default input device is available or an error was encountered\n");
+	return FALSE;
+      }
+    }
+  } else {
+    // use the given number as device id
+    devId = atoi(arg);
+  }
+  // output device information to use
   {
-    // choose a device to open
-    // For win32, preference order is WASAPI > ASIO > DirectSound > MME
-    // If several device matches the first found one will be used.
-    int devId;
-    PaDeviceIndex numDevice = Pa_GetDeviceCount(), i;
     const PaDeviceInfo *deviceInfo;
     const PaHostApiInfo *apiInfo;
-    PaStreamParameters param;
-    char *devname;
     static char buf[256];
-#ifdef _WIN32
-    // at win32, force preference order: iWASAPI > ASIO > DirectSound > MME > Other
-    int iMME = -1, iDS = -1, iASIO = -1, iWASAPI = -1;
-#endif
-
-	// if PORTAUDIO_DEV is specified, match it against available APIs
-    devname = getenv("PORTAUDIO_DEV");
-    devId = -1;
-
-	// get list of available capture devices
-    jlog("Stat: adin_portaudio: available capture devices:\n");
-    for(i=0;i<numDevice;i++) {
-      deviceInfo = Pa_GetDeviceInfo(i);
-      if (!deviceInfo) continue;
-      if (deviceInfo->maxInputChannels <= 0) continue;
-      apiInfo = Pa_GetHostApiInfo(deviceInfo->hostApi);
-      if (!apiInfo) continue;
-      snprintf(buf, 255, "%s: %s", apiInfo->name, deviceInfo->name);
-      buf[255] = '\0';
-      jlog("  #%d [%s]\n", i+1, buf);
-      if (devname && !strncmp(devname, buf, strlen(devname))) {
-		  // device name (partially) matches PORTAUDIO_DEV
-	devId = i;
-      }
-#ifdef _WIN32
-	  // store the device ID for each API
-      switch(apiInfo->type) {
-	  case paWASAPI: if (iWASAPI < 0) iWASAPI = i; break;
-      case paMME:if (iMME   < 0) iMME   = i; break;
-      case paDirectSound:if (iDS    < 0) iDS    = i; break;
-      case paASIO:if (iASIO  < 0) iASIO  = i; break;
-      }
-#endif
-    }
-    if (devId != -1) {
-      jlog("  --> #%d matches PORTAUDIO_DEV, use it\n", devId + 1);
-    }
-    if (devId == -1) {
-      if (getenv("PORTAUDIO_DEV_NUM")) {
-	devId = atoi(getenv("PORTAUDIO_DEV_NUM")) - 1;
-	jlog("  --> use #%d, specified by PORTAUDIO_DEV_NUM\n", devId + 1);
-      }
-    }
-#ifdef _WIN32
-    if (devId == -1) {
-      // if PORTAUDIO_DEV not specified or not matched, use device in the preference order.
-      if (iWASAPI >= 0) devId = iWASAPI;
-      else if (iASIO >= 0) devId = iASIO;
-      else if (iDS >= 0) devId = iDS;
-      else if (iMME >= 0) devId = iMME;
-      else devId = -1;
-    }
-#endif
-	if (devId == -1) {
-      // No device has been found, try to get the default input device
-      devId = Pa_GetDefaultInputDevice();
-	  if (devId == paNoDevice) {
-		  jlog("Error: adin_portaudio: no default input device is available or an error was encountered\n");
-		  return FALSE;
-	  }
-    }
-    // output device information
     deviceInfo = Pa_GetDeviceInfo(devId);
+    if (deviceInfo == NULL) {
+      jlog("Error: adin_portaudio: failed to get info for device id %d\n", devId);
+      return FALSE;
+    }
     apiInfo = Pa_GetHostApiInfo(deviceInfo->hostApi);
+    if (apiInfo == NULL) {
+      jlog("Error: adin_portaudio: failed to get API info for device id %d\n", devId);
+      return FALSE;
+    }
     snprintf(buf, 255, "%s: %s", apiInfo->name, deviceInfo->name);
     buf[255] = '\0';
-    jlog("Stat: adin_portaudio: get input from: [%s]\n", buf);
-    jlog("Info: adin_portaudio: set \"PORTAUDIO_DEV\" to the string in \"[]\" above to change\n");
-
-	// open the device
-      memset( &param, 0, sizeof(param));
-      param.channelCount = 1;
-      param.device = devId;
-      param.sampleFormat = paInt16;
-	  if (latency == 0) {
-	      param.suggestedLatency = Pa_GetDeviceInfo(devId)->defaultLowInputLatency;
-		  jlog("Stat: adin_portaudio: try to set default low latency from portaudio: %d msec\n", param.suggestedLatency * 1000.0);
-	  } else {
-	      param.suggestedLatency = latency / 1000.0;
-		  jlog("Stat: adin_portaudio: try to set latency to %d msec\n", param.suggestedLatency * 1000.0);
-	  }
-      err = Pa_OpenStream(&stream, &param, NULL, sfreq, 
-			  0, paNoFlag,
-			  Callback, NULL);
-      if (err != paNoError) {
-	jlog("Error: adin_portaudio: error in opening stream: %s\n", Pa_GetErrorText(err));
-	return(FALSE);
-      }
-    {
-      const PaStreamInfo *stinfo;
-      stinfo = Pa_GetStreamInfo(stream);
-      jlog("Stat: adin_portaudio: latency was set to %f msec\n", stinfo->inputLatency * 1000.0);
+    jlog("Stat: adin_portaudio: [%s]\n", buf);
+    jlog("Stat: adin_portaudio: (you can specify device by \"PORTAUDIO_DEV_NUM=number\"\n");
+  }
+  
+  // open the device
+  {
+    PaStreamParameters param;
+    memset( &param, 0, sizeof(param));
+    param.channelCount = 1;
+    param.device = devId;
+    param.sampleFormat = paInt16;
+    if (latency == 0) {
+      param.suggestedLatency = Pa_GetDeviceInfo(devId)->defaultLowInputLatency;
+      jlog("Stat: adin_portaudio: try to set default low latency from portaudio: %d msec\n", param.suggestedLatency * 1000.0);
+    } else {
+      param.suggestedLatency = latency / 1000.0;
+      jlog("Stat: adin_portaudio: try to set latency to %d msec\n", param.suggestedLatency * 1000.0);
     }
+    err = Pa_OpenStream(&stream, &param, NULL, sfreq, 
+			0, paNoFlag,
+			Callback, NULL);
+    if (err != paNoError) {
+      jlog("Error: adin_portaudio: error in opening stream: %s\n", Pa_GetErrorText(err));
+      return(FALSE);
+    }
+  }
+  {
+    const PaStreamInfo *stinfo;
+    stinfo = Pa_GetStreamInfo(stream);
+    jlog("Stat: adin_portaudio: latency was set to %f msec\n", stinfo->inputLatency * 1000.0);
   }
 
 #else
 
+  // Just open the default stream
   err = Pa_OpenDefaultStream(&stream, 1, 0, paInt16, sfreq, 
 #ifdef OLDVER
 			     frames_per_buffer,
@@ -400,14 +525,20 @@ adin_mic_standby(int sfreq, void *dummy)
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_mic_begin(char *pathname)
+adin_mic_begin(char *arg)
 {
   PaError err;
 
+  /* initialize device and open stream */
+  if (adin_mic_open(arg) == FALSE) {
+    stream = NULL;
+    return(FALSE);
+  }
   /* start stream */
   err = Pa_StartStream(stream);
   if (err != paNoError) {
     jlog("Error: adin_portaudio: failed to begin stream: %s\n", Pa_GetErrorText(err));
+    stream = NULL;
     return(FALSE);
   }
   
@@ -424,13 +555,29 @@ adin_mic_end()
 {
   PaError err;
 
-  /* stop stream */
-  err = Pa_StopStream(stream);
+  if (stream == NULL) return(TRUE);
+
+  /* stop stream (do not wait callback and buffer flush, stop immediately) */
+  err = Pa_AbortStream(stream);
   if (err != paNoError) {
     jlog("Error: adin_portaudio: failed to stop stream: %s\n", Pa_GetErrorText(err));
     return(FALSE);
   }
+  /* close stream */
+  err = Pa_CloseStream(stream);
+  if (err != paNoError) {
+    jlog("Error: adin_portaudio: failed to close stream: %s\n", Pa_GetErrorText(err));
+    return(FALSE);
+  }
+  /* terminate library */
+  err = Pa_Terminate();
+  if (err != paNoError) {
+    jlog("Error: adin_portaudio: failed to terminate library: %s\n", Pa_GetErrorText(err));
+    return(FALSE);
+  }
   
+  stream = NULL;
+
   return TRUE;
 }
 
@@ -463,6 +610,7 @@ adin_mic_read(SP16 *buf, int sampnum)
     printf("process  : current == processed: %d: wait\n", current);
 #endif
     Pa_Sleep(20); /* wait till some input comes */
+    if (stream == NULL) return(-1);
   }
 
   current_local = current;
