@@ -45,13 +45,13 @@
  * @author Akinobu LEE
  * @date   Mon Feb 14 14:55:03 2005
  *
- * $Revision: 1.3 $
+ * $Revision: 1.6 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -94,10 +94,12 @@ adin_tcpip_standby(int freq, void *port_str)
 /** 
  * Wait for connection from adinnet client and begin audio input stream.
  *
+ * @param pathname [in] path name to open or NULL for default
+ * 
  * @return TRUE on success, FALSE on failure.
  */
 boolean
-adin_tcpip_begin()
+adin_tcpip_begin(char *pathname)
 {
 #ifdef FORK_ADINNET
     /***********************************/
@@ -299,3 +301,15 @@ adin_tcpip_send_terminate()
    return TRUE;
 }
 
+/** 
+ * 
+ * Function to return current input source device name
+ * 
+ * @return string of current input device name.
+ * 
+ */
+char *
+adin_tcpip_input_name()
+{
+  return("network socket");
+}
