@@ -1,7 +1,5 @@
 /**
  * @file   vsegment.c
- * @author Akinobu LEE
- * @date   Fri Feb 18 19:29:22 2005
  * 
  * <JA>
  * @brief  ϤФViterbi 饤Ȥμ¹
@@ -11,13 +9,16 @@
  * @brief  Do viterbi alignment for the input
  * </EN>
  * 
- * $Revision: 1.2 $
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 19:29:22 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -37,6 +38,8 @@
  * 
  * @param hmm [in] sentence HMM to be matched
  * @param param [in] input parameter data
+ * @param wrk [i/o] HMM computation work area
+ * @param multipath [in] TRUE if need multi-path handling
  * @param endstates [in] list of state id that corrsponds to the ends of units
  * @param ulen [in] total number of units in the @a hmm
  * @param id_ret [out] Pointer to store the newly allocated array of the resulting id sequence of units on the best path.
@@ -47,15 +50,14 @@
  * @return the total acoustic score for the whole input.
  */
 LOGPROB
-viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_ret, int **seg_ret, LOGPROB **uscore_ret, int *slen_ret)
+viterbi_segment(HMM *hmm, HTK_Param *param, HMMWork *wrk, boolean multipath, int *endstates, int ulen, int **id_ret, int **seg_ret, LOGPROB **uscore_ret, int *slen_ret)
 {
   /* for viterbi */
   LOGPROB *nodescore[2];	/* node buffer */
   SEGTOKEN **tokenp[2];		/* propagating token which holds segment info */
+  int startt, endt;
   int *from_node;
-#ifndef MULTIPATH_VERSION
-  int *u_end, *u_start;	/* the node is an end of the word, or -1 */
-#endif
+  int *u_end, *u_start;	/* the node is an end of the word, or -1 for non-multipath mode*/
   int i, n;
   unsigned int t;
   int tl,tn;
@@ -70,30 +72,30 @@ viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_r
 
   /* assume more than 1 units */
   if (ulen < 1) {
-    j_printerr("Error: viterbi_segment: no unit?\n");
+    jlog("Error: vsegment: no unit?\n");
     return LOG_ZERO;
   }
 
-#ifndef MULTIPATH_VERSION
-  /* initialize unit start/end marker */
-  u_start = (int *)mymalloc(hmm->len * sizeof(int));
-  u_end   = (int *)mymalloc(hmm->len * sizeof(int));
-  for (n = 0; n < hmm->len; n++) {
-    u_start[n] = -1;
-    u_end[n] = -1;
-  }
-  u_start[0] = 0;
-  u_end[endstates[0]] = 0;
-  for (i=1;i<ulen;i++) {
-    u_start[endstates[i-1]+1] = i;
-    u_end[endstates[i]] = i;
-  }
+  if (!multipath) {
+    /* initialize unit start/end marker */
+    u_start = (int *)mymalloc(hmm->len * sizeof(int));
+    u_end   = (int *)mymalloc(hmm->len * sizeof(int));
+    for (n = 0; n < hmm->len; n++) {
+      u_start[n] = -1;
+      u_end[n] = -1;
+    }
+    u_start[0] = 0;
+    u_end[endstates[0]] = 0;
+    for (i=1;i<ulen;i++) {
+      u_start[endstates[i-1]+1] = i;
+      u_end[endstates[i]] = i;
+    }
 #if 0
-  for (i=0;i<hmm->len;i++) {
-    printf("unit %d: start=%d, end=%d\n", i, u_start[i], u_end[i]);
-  }
+    for (i=0;i<hmm->len;i++) {
+      printf("unit %d: start=%d, end=%d\n", i, u_start[i], u_end[i]);
+    }
 #endif
-#endif /* MULTIPATH_VERSION */
+  }
 
   /* initialize node buffers */
   tn = 0;
@@ -121,22 +123,21 @@ viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_r
   
   /* first frame: only set initial score */
   /*if (hmm->state[0].is_pseudo_state) {
-    j_printerr("Warning: state %d: pseudo state?\n", 0);
+    jlog("Warning: state %d: pseudo state?\n", 0);
     }*/
-#ifdef MULTIPATH_VERSION
-  nodescore[tn][0] = 0.0;
-#else
-  nodescore[tn][0] = outprob(0, &(hmm->state[0]), param);
-#endif
+  if (multipath) {
+    nodescore[tn][0] = 0.0;
+  } else {
+    nodescore[tn][0] = outprob(wrk, 0, &(hmm->state[0]), param);
+  }
 
   /* do viterbi for rest frame */
-  for (
-#ifdef MULTIPATH_VERSION
-       t = 0; t <= param->samplenum; t++
-#else
-       t = 1; t < param->samplenum; t++
-#endif
-       ) {
+  if (multipath) {
+    startt = 0;  endt = param->samplenum;
+  } else {
+    startt = 1;  endt = param->samplenum - 1;
+  }
+  for (t = startt; t <= endt; t++) {
     i = tl;
     tl = tn;
     tn = i;
@@ -161,54 +162,66 @@ viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_r
       }
     }
     /* propagate token, appending new if path was selected between units */
-    for (n = 0; n < hmm->len; n++) {
-#ifdef MULTIPATH_VERSION
-      if (from_node[n] == -1 || nodescore[tn][n] <= LOG_ZERO) {
-	/*tokenp[tn][n] = NULL;*/
-      } else {
-	i=0;
-	while (from_node[n] > endstates[i]) i++;
-	if (n > endstates[i]) {
-	  newtoken = (SEGTOKEN *)mymalloc(sizeof(SEGTOKEN));
-	  newtoken->last_id = i;
-#else
-      if (from_node[n] == -1) {
-	tokenp[tn][n] = NULL;
-      } else if (nodescore[tn][n] <= LOG_ZERO) {
-	tokenp[tn][n] = tokenp[tl][from_node[n]];
-      } else {
-	if (u_end[from_node[n]] != -1 && u_start[n] != -1
-	    && from_node[n] !=  n) {
-	  newtoken = (SEGTOKEN *)mymalloc(sizeof(SEGTOKEN));
-	  newtoken->last_id = u_end[from_node[n]];
-#endif
-	  newtoken->last_end_frame = t-1;
-	  newtoken->last_end_score = nodescore[tl][from_node[n]];
-	  newtoken->list = root;
-	  root = newtoken;
-	  newtoken->next = tokenp[tl][from_node[n]];
-	  tokenp[tn][n] = newtoken;
+    if (multipath) {
+      for (n = 0; n < hmm->len; n++) {
+	if (from_node[n] == -1 || nodescore[tn][n] <= LOG_ZERO) {
+	  /*tokenp[tn][n] = NULL;*/
 	} else {
+	  i=0;
+	  while (from_node[n] > endstates[i]) i++;
+	  if (n > endstates[i]) {
+	    newtoken = (SEGTOKEN *)mymalloc(sizeof(SEGTOKEN));
+	    newtoken->last_id = i;
+	    newtoken->last_end_frame = t-1;
+	    newtoken->last_end_score = nodescore[tl][from_node[n]];
+	    newtoken->list = root;
+	    root = newtoken;
+	    newtoken->next = tokenp[tl][from_node[n]];
+	    tokenp[tn][n] = newtoken;
+	  } else {
+	    tokenp[tn][n] = tokenp[tl][from_node[n]];
+	  }
+	}
+      }
+    } else {			/* not multipath */
+      for (n = 0; n < hmm->len; n++) {
+	if (from_node[n] == -1) {
+	  tokenp[tn][n] = NULL;
+	} else if (nodescore[tn][n] <= LOG_ZERO) {
 	  tokenp[tn][n] = tokenp[tl][from_node[n]];
+	} else {
+	  if (u_end[from_node[n]] != -1 && u_start[n] != -1
+	      && from_node[n] !=  n) {
+	    newtoken = (SEGTOKEN *)mymalloc(sizeof(SEGTOKEN));
+	    newtoken->last_id = u_end[from_node[n]];
+	    newtoken->last_end_frame = t-1;
+	    newtoken->last_end_score = nodescore[tl][from_node[n]];
+	    newtoken->list = root;
+	    root = newtoken;
+	    newtoken->next = tokenp[tl][from_node[n]];
+	    tokenp[tn][n] = newtoken;
+	  } else {
+	    tokenp[tn][n] = tokenp[tl][from_node[n]];
+	  }
 	}
       }
     }
 
-#ifdef MULTIPATH_VERSION
-    /* if this is next of last frame, loop ends here */
-    if (t == param->samplenum) break;
-#endif
+    if (multipath) {
+      /* if this is next of last frame, loop ends here */
+      if (t == param->samplenum) break;
+    }
 	
     /* calc outprob to new nodes */
     for (n = 0; n < hmm->len; n++) {
-#ifdef MULTIPATH_VERSION
-      if (hmm->state[n].out.state == NULL) continue;
-#endif
+      if (multipath) {
+	if (hmm->state[n].out.state == NULL) continue;
+      }
       if (nodescore[tn][n] > LOG_ZERO) {
 	if (hmm->state[n].is_pseudo_state) {
-	  j_printerr("Warning: state %d: pseudo state?\n", n);
+	  jlog("Warning: vsegment: state %d: pseudo state?\n", n);
 	}
-	nodescore[tn][n] += outprob(t, &(hmm->state[n]), param);
+	nodescore[tn][n] += outprob(wrk, t, &(hmm->state[n]), param);
       }
       if (nodescore[tn][n] > maxscore) { /* for debug */
 	maxscore = nodescore[tn][n];
@@ -220,10 +233,7 @@ viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_r
     for (i=0;i<ulen;i++) {
       printf("%d: unit %d(%d-%d): begin_frame = %d\n", t - 1, i,
 	     (i > 0) ? endstates[i-1]+1 : 0, endstates[i],
-#ifdef MULTIPATH_VERSION
-	     (tokenp[tl][endstates[i]] == NULL) ? -1 :
-#endif
-	     tokenp[tl][endstates[i]]->last_end_frame + 1);
+	     (multipath && tokenp[tl][endstates[i]] == NULL) ? -1 : tokenp[tl][endstates[i]]->last_end_frame + 1);
     }
 #endif
 
@@ -235,11 +245,8 @@ viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_r
 
   /* parse back the last token to see the trail of best viterbi path */
   /* and store the informations to returning buffer */
-#ifdef MULTIPATH_VERSION
   slen = 0;
-#else
-  slen = 1;
-#endif
+  if (!multipath) slen++;
   for(token = tokenp[tn][hmm->len-1]; token; token = token->next) {
     if (token->last_end_frame == -1) break;
     slen++;
@@ -248,14 +255,14 @@ viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_r
   seg = (int *)mymalloc(sizeof(int)*slen);
   uscore = (LOGPROB *)mymalloc(sizeof(LOGPROB)*slen);
 
-#ifdef MULTIPATH_VERSION
-  i = slen - 1;
-#else
-  id[slen-1] = ulen - 1;
-  seg[slen-1] = t - 1;
-  uscore[slen-1] = result_score;
-  i = slen - 2;
-#endif
+  if (multipath) {
+    i = slen - 1;
+  } else {
+    id[slen-1] = ulen - 1;
+    seg[slen-1] = t - 1;
+    uscore[slen-1] = result_score;
+    i = slen - 2;
+  }
   for(token = tokenp[tn][hmm->len-1]; token; token = token->next) {
     if (i < 0 || token->last_end_frame == -1) break;
     id[i] = token->last_id;
@@ -277,10 +284,10 @@ viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_r
   *slen_ret = slen;
 
   /* free memory */
-#ifndef MULTIPATH_VERSION
-  free(u_start);
-  free(u_end);
-#endif
+  if (!multipath) {
+    free(u_start);
+    free(u_end);
+  }
   free(from_node);
   token = root;
   while(token) {
