@@ -1,7 +1,5 @@
 /**
  * @file   mystrtok.c
- * @author Akinobu LEE
- * @date   Thu Feb 17 16:31:39 2005
  * 
  * <JA>
  * @brief  ʸ򥯥ơθƥȡʬ䤹
@@ -17,13 +15,16 @@
  * braces are forced to be treated as a single token.
  * </EN>
  * 
- * $Revision: 1.2 $
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 16:31:39 2005
+ *
+ * $Revision: 1.3 $
  * 
  */
 /*
- * Copyright (c) 1991-2006 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2006 Julius project team, Nagoya Institute of Technology, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -61,7 +62,7 @@ mystrtok_quotation(char *str, char *delim, int left_paren, int right_paren, int
 
   /* find start point */
   p = pos;
-  while ((c = *p) != '\0' && ISTOKEN(c)) *p++;
+  while (*p != '\0' && ISTOKEN(*p)) p++;
   if (*p == '\0') return NULL;	/* no token left */
 
   /* if mode == 1, exit here */
@@ -76,7 +77,9 @@ mystrtok_quotation(char *str, char *delim, int left_paren, int right_paren, int
     p++;
     if (*p == '\0') return NULL;
     from = p;
-    while ((c = *p) != '\0' && (c != right_paren)) p++;
+    while ((c = *p) != '\0' && 
+	   ((c != right_paren) || (*(p+1) != '\0' && !ISTOKEN(*(p+1))))) p++;
+	
     /* if quotation not terminated, allow the rest as one token */
     /* if (*p == '\0') return NULL; */
   } else {
