@@ -19,13 +19,13 @@
  * @author Akinobu LEE
  * @date   Fri Feb 18 19:43:06 2005
  *
- * $Revision: 1.7 $
+ * $Revision: 1.11 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -62,8 +62,8 @@
 static char buf[MAXLINELEN];	///< Local work area for input text processing
 static char bufbak[MAXLINELEN];	///< Local work area for debug message
 
-static char trbuf[3][20];	///< Local buffer for triphone convertion
-static char chbuf[30];	     ///< Another local buffer for triphone convertion
+static char trbuf[3][MAX_HMMNAME_LEN];	///< Local buffer for triphone convertion
+static char chbuf[MAX_HMMNAME_LEN];	     ///< Another local buffer for triphone convertion
 static char nophone[1];		///< Local buffer to indicate 'no phone'
 static int  trp_l;		///< Triphone cycle index
 static int  trp;		///< Triphone cycle index
@@ -375,8 +375,7 @@ voca_load_htkdict_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winfo
 {
   char *ptmp, *lp = NULL, *p;
   static char cbuf[MAX_HMMNAME_LEN];
-  static HMM_Logical **tmpwseq = NULL;
-  static int tmpmaxlen;
+  HMM_Logical **tmpwseq;
   int len;
   HMM_Logical *tmplg;
   boolean pok;
@@ -387,10 +386,11 @@ voca_load_htkdict_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winfo
   if (strmatch(buf, "DICEND")) return FALSE;
 
   /* allocate temporal work area for the first call */
-  if (tmpwseq == NULL) {
-    tmpmaxlen = PHONEMELEN_STEP;
-    tmpwseq = (HMM_Logical **)mymalloc(sizeof(HMM_Logical *) * tmpmaxlen);
+  if (winfo->work == NULL) {
+    winfo->work_num = PHONEMELEN_STEP;
+    winfo->work = (void *)mybmalloc2(sizeof(HMM_Logical *) * winfo->work_num, &(winfo->mroot));
   }
+  tmpwseq = (HMM_Logical **)winfo->work;
 
   /* backup whole line for debug output */
   strcpy(bufbak, buf);
@@ -509,6 +509,12 @@ voca_load_htkdict_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winfo
 	*ok_flag = FALSE;
 	return TRUE;
       }
+      if (strlen(lp) >= MAX_HMMNAME_LEN) {
+	jlog("Error: voca_load_htkdict: line %d: too long phone name: %s\n", linenum, lp);
+	winfo->errnum++;
+	*ok_flag = FALSE;
+	return TRUE;
+      }
       cycle_triphone(lp);
     }
 
@@ -517,7 +523,15 @@ voca_load_htkdict_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winfo
       if (do_conv) {
 /*	if (lp != NULL) jlog(" %d%s",len,lp);*/
 	if (lp != NULL) lp = mystrtok(NULL, " \t\n");
-	if (lp != NULL) p = cycle_triphone(lp);
+	if (lp != NULL) {
+	  if (strlen(lp) >= MAX_HMMNAME_LEN) {
+	    jlog("Error: voca_load_htkdict: line %d: too long phone name: %s\n", linenum, lp);
+	    winfo->errnum++;
+	    *ok_flag = FALSE;
+	    return TRUE;
+	  }
+	  p = cycle_triphone(lp);
+	}
 	else p = cycle_triphone_flush();
       } else {
 	p = mystrtok(NULL, " \t\n");
@@ -551,10 +565,12 @@ voca_load_htkdict_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winfo
 	pok = FALSE;
       } else {
 	/* found */
-	if (len >= tmpmaxlen) {
+	if (len >= winfo->work_num) {
 	  /* expand wseq area by PHONEMELEN_STEP */
-	  tmpmaxlen += PHONEMELEN_STEP;
-	  tmpwseq = (HMM_Logical **)myrealloc(tmpwseq, sizeof(HMM_Logical *) * tmpmaxlen);
+	  winfo->work_num += PHONEMELEN_STEP;
+	  winfo->work = (void *)mybmalloc2(sizeof(HMM_Logical *) * winfo->work_num, &(winfo->mroot));
+	  memcpy(winfo->work, tmpwseq, sizeof(HMM_Logical *) * (winfo->work_num - PHONEMELEN_STEP));
+	  tmpwseq = (HMM_Logical **)winfo->work;
 	}
 	/* store to temporal buffer */
 	tmpwseq[len] = tmplg;
@@ -651,12 +667,28 @@ voca_append(WORD_INFO *dstinfo, WORD_INFO *srcinfo, int coffset, int woffset)
   for(w=0;w<srcinfo->num;w++) {
     /* copy data */
     dstinfo->wlen[n] = srcinfo->wlen[w];
-    if (srcinfo->wname[w]) dstinfo->wname[n] = strcpy((char *)mybmalloc2(strlen(srcinfo->wname[w])+1, &(dstinfo->mroot)), srcinfo->wname[w]);
-    if (srcinfo->woutput[w]) dstinfo->woutput[n] = strcpy((char *)mybmalloc2(strlen(srcinfo->woutput[w])+1, &(dstinfo->mroot)), srcinfo->woutput[w]);
-    if (srcinfo->wlen[w] > 0) dstinfo->wseq[n] = (HMM_Logical **)mybmalloc2(sizeof(HMM_Logical *) * srcinfo->wlen[w], &(dstinfo->mroot));
-    for(i=0;i<srcinfo->wlen[w];i++) {
-      dstinfo->wseq[n][i] = srcinfo->wseq[w][i];
+    if (srcinfo->wname[w]) {
+      dstinfo->wname[n] = strcpy((char *)mybmalloc2(strlen(srcinfo->wname[w])+1, &(dstinfo->mroot)), srcinfo->wname[w]);
+    } else {
+      dstinfo->wname[n] = NULL;
     }
+    if (srcinfo->woutput[w]) {
+      dstinfo->woutput[n] = strcpy((char *)mybmalloc2(strlen(srcinfo->woutput[w])+1, &(dstinfo->mroot)), srcinfo->woutput[w]);
+    } else {
+      dstinfo->woutput[n] = NULL;
+    }
+    if (srcinfo->wlen[w] > 0) {
+      dstinfo->wseq[n] = (HMM_Logical **)mybmalloc2(sizeof(HMM_Logical *) * srcinfo->wlen[w], &(dstinfo->mroot));
+      for(i=0;i<srcinfo->wlen[w];i++) {
+	dstinfo->wseq[n][i] = srcinfo->wseq[w][i];
+      }
+    } else {
+      dstinfo->wseq[n] = NULL;
+    }
+#ifdef CLASS_NGRAM
+    dstinfo->cprob[n] = srcinfo->cprob[w];
+    if (dstinfo->cprob[n] != 0.0) dstinfo->cwnum++;
+#endif
     dstinfo->is_transparent[n] = srcinfo->is_transparent[w];
     /* offset category ID by coffset */
     dstinfo->wton[n] = srcinfo->wton[w] + coffset;
