@@ -13,13 +13,13 @@
  * @author Akinobu LEE
  * @date   Sun Jul 22 13:29:32 2007
  *
- * $Revision: 1.4 $
+ * $Revision: 1.7 $
  * 
  */
 /*
- * Copyright (c) 1991-2007 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2011 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2007 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2011 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -127,7 +127,7 @@ voca_load_word_line(char *buf, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, char *he
   return TRUE;
 }
 /** 
- * Top function to read word list via file pointer
+ * Top function to read word list via text
  * 
  * @param fp [in] file pointer
  * @param winfo [out] pointer to word dictionary to store the read data.
@@ -152,6 +152,33 @@ voca_load_wordlist(FILE *fp, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, char *head
   return(ret);
 }
 
+
+/** 
+ * Top function to read word list via file pointer
+ * 
+ * @param fp [in] file pointer
+ * @param winfo [out] pointer to word dictionary to store the read data.
+ * @param hmminfo [in] HTK %HMM definition data.  if NULL, phonemes are ignored.
+ * @param headphone [in] word head silence model name
+ * @param tailphone [in] word tail silence model name
+ * @param contextphone [in] silence context name to be used at head and tail
+ * 
+ * @return TRUE on success, FALSE on any error word.
+ */
+boolean
+voca_load_wordlist_fp(FILE *fp, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo, char *headphone, char *tailphone, char *contextphone)
+{
+  boolean ret;
+
+  voca_load_start(winfo, hmminfo, FALSE);
+  while (getl_fp(buf, sizeof(buf), fp) != NULL) {
+    if (voca_load_word_line(buf, winfo, hmminfo, headphone, tailphone, contextphone) == FALSE) break;
+  }
+  ret = voca_load_end(winfo);
+
+  return(ret);
+}
+
 /** 
  * Sub function to Add a dictionary entry line to the word dictionary.
  * 
@@ -298,6 +325,12 @@ voca_load_wordlist_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winf
 	  cycle_triphone(NULL);
 	  /* insert head phone at beginning of word */
 	  if (contextphone) {
+	    if (strlen(contextphone) >= MAX_HMMNAME_LEN) {
+	      jlog("Error: voca_load_htkdict: line %d: too long phone name: %s\n", linenum, contextphone);
+	      winfo->errnum++;
+	      *ok_flag = FALSE;
+	      return TRUE;
+	    }
 	    cycle_triphone(contextphone);
 	  } else {
 	    cycle_triphone("NULL_C");
@@ -308,6 +341,12 @@ voca_load_wordlist_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winf
 	    *ok_flag = FALSE;
 	    return TRUE;
 	  }
+	  if (strlen(lp) >= MAX_HMMNAME_LEN) {
+	    jlog("Error: voca_load_htkdict: line %d: too long phone name: %s\n", linenum, lp);
+	    winfo->errnum++;
+	    *ok_flag = FALSE;
+	    return TRUE;
+	  }
 	  p = cycle_triphone(lp);
 	  first = FALSE;
 	} else {		/* do_conv, not first */
@@ -315,10 +354,22 @@ voca_load_wordlist_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winf
 	    lp = mystrtok(NULL, " \t\n");
 	    if (lp != NULL) {
 	      /* token exist */
+	      if (strlen(lp) >= MAX_HMMNAME_LEN) {
+		jlog("Error: voca_load_htkdict: line %d: too long phone name: %s\n", linenum, lp);
+		winfo->errnum++;
+		*ok_flag = FALSE;
+		return TRUE;
+	      }
 	      p = cycle_triphone(lp);
 	    } else {
 	      /* no more token, insert tail phone at end of word */
 	      if (contextphone) {
+		if (strlen(contextphone) >= MAX_HMMNAME_LEN) {
+		  jlog("Error: voca_load_htkdict: line %d: too long phone name: %s\n", linenum, contextphone);
+		  winfo->errnum++;
+		  *ok_flag = FALSE;
+		  return TRUE;
+		}
 		p = cycle_triphone(contextphone);
 	      } else {
 		p = cycle_triphone("NULL_C");
@@ -421,6 +472,7 @@ voca_load_wordlist_line(char *buf, WORD_ID *vnum_p, int linenum, WORD_INFO *winf
     winfo->wseq[vnum] = (HMM_Logical **)mybmalloc2(sizeof(HMM_Logical *) * len, &(winfo->mroot));
     memcpy(winfo->wseq[vnum], tmpwseq, sizeof(HMM_Logical *) * len);
     winfo->wlen[vnum] = len;
+    winfo->wton[vnum] = 0;
   }
 
   vnum++;
